#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include "error.h"

static __thread AL_ERROR_ENUM _al_errcode;
static __thread const char *_al_errstr;

static __thread void (*_allegro_error)(void) = AL_ERROR_ABORT;


void al_error_set_handler(void (*func)(void))
{
   _allegro_error = func;
}


void AL_ERROR_ABORT(void)
{
   const char *str;
   AL_ERROR_ENUM code;

   code = al_error(&str);

   fprintf(stderr, "An error occured!\nCode: %d\nMessage: %s\nAborting\n", code, str);
   abort();
}

void AL_ERROR_WARN(void)
{
   const char *str;
   AL_ERROR_ENUM code;

   code = al_error(&str);
   fprintf(stderr, "An error occured!\nCode: %d\nMessage: %s\n", code, str);

   al_error_set(code, str);
}


void al_error_set(AL_ERROR_ENUM code, const char *str)
{
	_al_errcode = code;
	if(!str)
	{
		switch(code)
		{
#define CASE(x)	case x:	_al_errstr = #x; break
			CASE(AL_NO_ERROR);
			CASE(AL_INVALID_PARAM);
			CASE(AL_INVALID_OBJECT);
			CASE(AL_GENERIC_ERROR);
#undef CASE
			default:
				_al_errstr = "Unknown error";
				break;
		}
	}
	else
		_al_errstr = str;

	if(_allegro_error)
		_allegro_error();
}

AL_ERROR_ENUM al_error(const char **str)
{
	AL_ERROR_ENUM ret = _al_errcode;
	if(str)
		*str = _al_errstr;

	_al_errcode = AL_NO_ERROR;
	_al_errstr = NULL;

	return ret;
}
